@@ -66,6 +66,9 @@ static const char *cpinr_prefixes[] = { "+CPINR:", "+CPINRE:", NULL };
 static const char *epin_prefix[] = { "*EPIN:", NULL };
 static const char *spic_prefix[] = { "+SPIC:", NULL };
 static const char *pct_prefix[] = { "#PCT:", NULL };
+static const char *pnnm_prefix[] = { "+PNNM:", NULL };
+static const char *qpinc_prefix[] = { "+QPINC:", NULL };
+static const char *upincnt_prefix[] = { "+UPINCNT:", NULL };
 static const char *none_prefix[] = { NULL };
 
 static void at_crsm_info_cb(gboolean ok, GAtResult *result, gpointer user_data)
@@ -132,12 +135,14 @@ error:
 }
 
 static void at_sim_read_info(struct ofono_sim *sim, int fileid,
-					ofono_sim_file_info_cb_t cb,
-					void *data)
+				const unsigned char *path,
+				unsigned int path_len,
+				ofono_sim_file_info_cb_t cb, void *data)
 {
 	struct sim_data *sd = ofono_sim_get_data(sim);
 	struct cb_data *cbd;
-	char buf[64];
+	char buf[128];
+	unsigned int len;
 
 	if (sd->vendor == OFONO_VENDOR_OPTION_HSO) {
 		unsigned char access[3] = { 0x00, 0x00, 0x00 };
@@ -151,18 +156,35 @@ static void at_sim_read_info(struct ofono_sim *sim, int fileid,
 
 	cbd = cb_data_new(cb, data);
 
-	snprintf(buf, sizeof(buf), "AT+CRSM=192,%i", fileid);
+	len = snprintf(buf, sizeof(buf), "AT+CRSM=192,%i", fileid);
 
 	switch (sd->vendor) {
+	default:
+		if (path_len == 0)
+			break;
+
+		/* Fall through */
 	case OFONO_VENDOR_ZTE:
 	case OFONO_VENDOR_HUAWEI:
 	case OFONO_VENDOR_SIERRA:
 	case OFONO_VENDOR_SPEEDUP:
 	case OFONO_VENDOR_QUALCOMM_MSM:
-		strcat(buf, ",0,0,255"); /* Maximum possible length */
+	case OFONO_VENDOR_SIMCOM:
+		/* Maximum possible length */
+		len += sprintf(buf + len, ",0,0,255");
 		break;
 	}
 
+	if (path_len > 0) {
+		len += sprintf(buf + len, ",,\"");
+
+		for (; path_len; path_len--)
+			len += sprintf(buf + len, "%02hhX", *path++);
+
+		buf[len++] = '\"';
+		buf[len] = '\0';
+	}
+
 	if (g_at_chat_send(sd->chat, buf, crsm_prefix,
 				at_crsm_info_cb, cbd, g_free) > 0)
 		return;
@@ -222,16 +244,31 @@ static void at_crsm_read_cb(gboolean ok, GAtResult *result,
 }
 
 static void at_sim_read_binary(struct ofono_sim *sim, int fileid,
-					int start, int length,
-					ofono_sim_read_cb_t cb, void *data)
+				int start, int length,
+				const unsigned char *path,
+				unsigned int path_len,
+				ofono_sim_read_cb_t cb, void *data)
 {
 	struct sim_data *sd = ofono_sim_get_data(sim);
 	struct cb_data *cbd = cb_data_new(cb, data);
 	char buf[64];
+	unsigned int len;
 
-	snprintf(buf, sizeof(buf), "AT+CRSM=176,%i,%i,%i,%i", fileid,
+	len = snprintf(buf, sizeof(buf), "AT+CRSM=176,%i,%i,%i,%i", fileid,
 			start >> 8, start & 0xff, length);
 
+	if (path_len > 0) {
+		buf[len++] = ',';
+		buf[len++] = ',';
+		buf[len++] = '\"';
+
+		for (; path_len; path_len--)
+			len += sprintf(buf + len, "%02hhX", *path++);
+
+		buf[len++] = '\"';
+		buf[len] = '\0';
+	}
+
 	if (g_at_chat_send(sd->chat, buf, crsm_prefix,
 				at_crsm_read_cb, cbd, g_free) > 0)
 		return;
@@ -242,12 +279,14 @@ static void at_sim_read_binary(struct ofono_sim *sim, int fileid,
 }
 
 static void at_sim_read_record(struct ofono_sim *sim, int fileid,
-					int record, int length,
-					ofono_sim_read_cb_t cb, void *data)
+				int record, int length,
+				const unsigned char *path,
+				unsigned int path_len,
+				ofono_sim_read_cb_t cb, void *data)
 {
 	struct sim_data *sd = ofono_sim_get_data(sim);
 	struct cb_data *cbd = cb_data_new(cb, data);
-	char buf[64];
+	char buf[128];
 
 	snprintf(buf, sizeof(buf), "AT+CRSM=178,%i,%i,4,%i", fileid,
 			record, length);
@@ -303,42 +342,29 @@ static void at_crsm_update_cb(gboolean ok, GAtResult *result,
 static void at_sim_update_file(struct ofono_sim *sim, int cmd, int fileid,
 				int p1, int p2, int p3,
 				const unsigned char *value,
+				const unsigned char *path,
+				unsigned int path_len,
 				ofono_sim_write_cb_t cb, void *data)
 {
 	struct sim_data *sd = ofono_sim_get_data(sim);
 	struct cb_data *cbd = cb_data_new(cb, data);
 	char *buf;
-	gboolean quote = FALSE;
 	int len, ret;
-	int size = 36 + p3 * 2;
+	int size = 38 + p3 * 2;
 
 	DBG("");
 
-	/* Add quotes */
-	switch (sd->vendor) {
-	case OFONO_VENDOR_MBM:
-	case OFONO_VENDOR_ZTE:
-	case OFONO_VENDOR_HUAWEI:
-	case OFONO_VENDOR_SPEEDUP:
-		quote = TRUE;
-		size += 2;
-		break;
-	}
-
 	buf = g_try_new(char, size);
 	if (buf == NULL)
 		goto error;
 
-	len = sprintf(buf, "AT+CRSM=%i,%i,%i,%i,%i,", cmd, fileid,p1, p2, p3);
-
-	if (quote)
-		buf[len++] = '\"';
+	len = sprintf(buf, "AT+CRSM=%i,%i,%i,%i,%i,\"", cmd, fileid,p1, p2, p3);
 
 	for (; p3; p3--)
 		len += sprintf(buf + len, "%02hhX", *value++);
 
-	if (quote)
-		buf[len++] = '\"';
+	buf[len++] = '\"';
+	buf[len] = '\0';
 
 	ret = g_at_chat_send(sd->chat, buf, crsm_prefix,
 				at_crsm_update_cb, cbd, g_free);
@@ -357,26 +383,33 @@ error:
 static void at_sim_update_binary(struct ofono_sim *sim, int fileid,
 					int start, int length,
 					const unsigned char *value,
+					const unsigned char *path,
+					unsigned int path_len,
 					ofono_sim_write_cb_t cb, void *data)
 {
 	at_sim_update_file(sim, 214, fileid, start >> 8, start & 0xff,
-				length, value, cb, data);
+				length, value, path, path_len, cb, data);
 }
 
 static void at_sim_update_record(struct ofono_sim *sim, int fileid,
 					int record, int length,
 					const unsigned char *value,
+					const unsigned char *path,
+					unsigned int path_len,
 					ofono_sim_write_cb_t cb, void *data)
 {
 	at_sim_update_file(sim, 220, fileid, record, 4, length,
-				value, cb, data);
+				value, path, path_len, cb, data);
 }
 
 static void at_sim_update_cyclic(struct ofono_sim *sim, int fileid,
 					int length, const unsigned char *value,
+					const unsigned char *path,
+					unsigned int path_len,
 					ofono_sim_write_cb_t cb, void *data)
 {
-	at_sim_update_file(sim, 220, fileid, 0, 3, length, value, cb, data);
+	at_sim_update_file(sim, 220, fileid, 0, 3, length, value,
+				path, path_len, cb, data);
 }
 
 static void at_cimi_cb(gboolean ok, GAtResult *result, gpointer user_data)
@@ -899,6 +932,127 @@ error:
 	CALLBACK_WITH_FAILURE(cb, NULL, cbd->data);
 }
 
+static void at_pnnm_cb(gboolean ok, GAtResult *result, gpointer user_data)
+{
+	struct cb_data *cbd = user_data;
+	ofono_sim_pin_retries_cb_t cb = cbd->cb;
+	const char *final = g_at_result_final_response(result);
+	GAtResultIter iter;
+	struct ofono_error error;
+	int retries[OFONO_SIM_PASSWORD_INVALID];
+	size_t i;
+	static enum ofono_sim_password_type password_types[] = {
+		OFONO_SIM_PASSWORD_SIM_PIN,
+		OFONO_SIM_PASSWORD_SIM_PUK,
+	};
+
+	decode_at_error(&error, final);
+
+	if (!ok) {
+		cb(&error, NULL, cbd->data);
+		return;
+	}
+
+	g_at_result_iter_init(&iter, result);
+
+	if (!g_at_result_iter_next(&iter, "+PNNM:"))
+		goto error;
+
+	BUILD_PIN_RETRIES_ARRAY(password_types, ARRAY_SIZE(password_types),
+				retries);
+
+	cb(&error, retries, cbd->data);
+
+	return;
+
+error:
+	CALLBACK_WITH_FAILURE(cb, NULL, cbd->data);
+}
+
+static void at_qpinc_cb(gboolean ok, GAtResult *result, gpointer user_data)
+{
+	struct cb_data *cbd = user_data;
+	ofono_sim_pin_retries_cb_t cb = cbd->cb;
+	const char *final = g_at_result_final_response(result);
+	GAtResultIter iter;
+	struct ofono_error error;
+	int retries[OFONO_SIM_PASSWORD_INVALID];
+	size_t i;
+
+	decode_at_error(&error, final);
+
+	if (!ok) {
+		cb(&error, NULL, cbd->data);
+		return;
+	}
+
+	for (i = 0; i < OFONO_SIM_PASSWORD_INVALID; i++)
+		retries[i] = -1;
+
+	g_at_result_iter_init(&iter, result);
+
+	while (g_at_result_iter_next(&iter, "+QPINC:")) {
+		const char *name;
+		int pin, puk;
+
+		if (!g_at_result_iter_next_string(&iter, &name))
+			continue;
+		if (!g_at_result_iter_next_number(&iter, &pin))
+			continue;
+		if (!g_at_result_iter_next_number(&iter, &puk))
+			continue;
+
+		if (!strcmp(name, "SC")) {
+			retries[OFONO_SIM_PASSWORD_SIM_PIN] = pin;
+			retries[OFONO_SIM_PASSWORD_SIM_PUK] = puk;
+		} else if (!strcmp(name, "P2")) {
+			retries[OFONO_SIM_PASSWORD_SIM_PIN2] = pin;
+			retries[OFONO_SIM_PASSWORD_SIM_PUK2] = puk;
+		}
+	}
+
+	cb(&error, retries, cbd->data);
+}
+
+static void upincnt_cb(gboolean ok, GAtResult *result, gpointer user_data)
+{
+	struct cb_data *cbd = user_data;
+	ofono_sim_pin_retries_cb_t cb = cbd->cb;
+	const char *final = g_at_result_final_response(result);
+	GAtResultIter iter;
+	struct ofono_error error;
+	int retries[OFONO_SIM_PASSWORD_INVALID];
+	size_t i;
+	static enum ofono_sim_password_type password_types[] = {
+		OFONO_SIM_PASSWORD_SIM_PIN,
+		OFONO_SIM_PASSWORD_SIM_PIN2,
+		OFONO_SIM_PASSWORD_SIM_PUK,
+		OFONO_SIM_PASSWORD_SIM_PUK2,
+	};
+
+	decode_at_error(&error, final);
+
+	if (!ok) {
+		cb(&error, NULL, cbd->data);
+		return;
+	}
+
+	g_at_result_iter_init(&iter, result);
+
+	if (!g_at_result_iter_next(&iter, "+UPINCNT:"))
+		goto error;
+
+	BUILD_PIN_RETRIES_ARRAY(password_types, ARRAY_SIZE(password_types),
+				retries);
+
+	cb(&error, retries, cbd->data);
+
+	return;
+
+error:
+	CALLBACK_WITH_FAILURE(cb, NULL, cbd->data);
+}
+
 static void at_pin_retries_query(struct ofono_sim *sim,
 					ofono_sim_pin_retries_cb_t cb,
 					void *data)
@@ -955,6 +1109,21 @@ static void at_pin_retries_query(struct ofono_sim *sim,
 					at_pct_cb, cbd, g_free) > 0)
 			return;
 		break;
+	case OFONO_VENDOR_ALCATEL:
+		if (g_at_chat_send(sd->chat, "AT+PNNM?", pnnm_prefix,
+					at_pnnm_cb, cbd, g_free) > 0)
+			return;
+		break;
+	case OFONO_VENDOR_QUECTEL:
+		if (g_at_chat_send(sd->chat, "AT+QPINC?", qpinc_prefix,
+					at_qpinc_cb, cbd, g_free) > 0)
+			return;
+		break;
+	case OFONO_VENDOR_UBLOX:
+		if (g_at_chat_send(sd->chat, "AT+UPINCNT", upincnt_prefix,
+					upincnt_cb, cbd, g_free) > 0)
+			return;
+		break;
 	default:
 		if (g_at_chat_send(sd->chat, "AT+CPINR", cpinr_prefixes,
 					at_cpinr_cb, cbd, g_free) > 0)
@@ -967,6 +1136,12 @@ static void at_pin_retries_query(struct ofono_sim *sim,
 	CALLBACK_WITH_FAILURE(cb, NULL, data);
 }
 
+static int needs_wavecom_sim_quirk(int vendor)
+{
+	return vendor == OFONO_VENDOR_WAVECOM ||
+			vendor == OFONO_VENDOR_WAVECOM_Q2XXX;
+}
+
 static void at_cpin_cb(gboolean ok, GAtResult *result, gpointer user_data)
 {
 	struct cb_data *cbd = user_data;
@@ -980,7 +1155,7 @@ static void at_cpin_cb(gboolean ok, GAtResult *result, gpointer user_data)
 	int len = sizeof(at_sim_name) / sizeof(*at_sim_name);
 	const char *final = g_at_result_final_response(result);
 
-	if (sd->vendor == OFONO_VENDOR_WAVECOM && ok && strlen(final) > 7)
+	if (needs_wavecom_sim_quirk(sd->vendor) && ok && strlen(final) > 7)
 		decode_at_error(&error, "OK");
 	else
 		decode_at_error(&error, final);
@@ -990,8 +1165,7 @@ static void at_cpin_cb(gboolean ok, GAtResult *result, gpointer user_data)
 		return;
 	}
 
-	if (sd->vendor == OFONO_VENDOR_WAVECOM ||
-			sd->vendor == OFONO_VENDOR_WAVECOM_Q2XXX) {
+	if (needs_wavecom_sim_quirk(sd->vendor)) {
 		/* +CPIN: <pin> */
 		pin_required = final + 7;
 	} else {
@@ -1059,7 +1233,6 @@ static void at_xsim_notify(GAtResult *result, gpointer user_data)
 
 	switch (state) {
 	case 3:	/* PIN verified – Ready */
-	case 7:	/* ready for attach (+COPS) */
 		break;
 	default:
 		return;
@@ -1102,7 +1275,7 @@ static void at_qss_notify(GAtResult *result, gpointer user_data)
 		return;
 
 	switch (state) {
-	case 2:	/* PIN unlocked */
+	case 3:	/* SIM inserted and READY. */
 		break;
 	default:
 		return;
@@ -1174,10 +1347,19 @@ static void at_pin_send_cb(gboolean ok, GAtResult *result,
 							FALSE, cbd, g_free);
 		return;
 	case OFONO_VENDOR_ZTE:
+	case OFONO_VENDOR_ALCATEL:
+	case OFONO_VENDOR_HUAWEI:
+	case OFONO_VENDOR_SIMCOM:
 		/*
 		 * On ZTE modems, after pin is entered, SIM state is checked
 		 * by polling CPIN as their modem doesn't provide unsolicited
 		 * notification of SIM readiness.
+		 *
+		 * On SIMCOM modems, SIM is busy after pin is entered (we
+		 * got a "+CME ERROR: 14" for the "AT+CPIN?" request) and
+		 * ofono don't catch the "+CPIN: READY" message sent by the
+		 * modem when SIM is ready. So, use extra CPIN to check the
+		 * state.
 		 */
 		sd->sim_state_query = at_util_sim_state_query_new(sd->chat,
 						2, 20, sim_state_cb, cbd,
@@ -1402,19 +1584,8 @@ static int at_sim_probe(struct ofono_sim *sim, unsigned int vendor,
 	sd->chat = g_at_chat_clone(chat);
 	sd->vendor = vendor;
 
-	switch (sd->vendor) {
-	case OFONO_VENDOR_WAVECOM:
-		g_at_chat_add_terminator(sd->chat, "+CPIN:", 6, TRUE);
-		break;
-	case OFONO_VENDOR_MBM:
+	if (sd->vendor == OFONO_VENDOR_MBM)
 		g_at_chat_send(sd->chat, "AT*EPEE=1", NULL, NULL, NULL, NULL);
-		break;
-	case OFONO_VENDOR_WAVECOM_Q2XXX:
-		g_at_chat_add_terminator(chat, "+CPIN: READY", -1, TRUE);
-		break;
-	default:
-		break;
-	}
 
 	ofono_sim_set_data(sim, sd);
 	g_idle_add(at_sim_register, sim);
@@ -1426,6 +1597,7 @@ static void at_sim_remove(struct ofono_sim *sim)
 {
 	struct sim_data *sd = ofono_sim_get_data(sim);
 
+	g_idle_remove_by_data(sim);
 	/* Cleanup potential SIM state polling */
 	at_util_sim_state_query_free(sd->sim_state_query);
 
